<template>
  <div class="chart-context-menu" v-show="visible">
    <!-- mask -->
    <div
      class="mask"
      v-show="visible"
      @click="close"
      @contextmenu="close"
    ></div>
    <div class="contain">
      <ul class="main-ul">
        <li
          class="main-li"
          v-for="(item, idx) in menuList"
          :key="'main_' + idx"
        >
          <div :class="['title', {disabled: item.disabled}]" @click="itemClick(item)">
            {{ item.title }}
          </div>
        </li>
      </ul>
    </div>
  </div>
</template>

<script>
export default {
  name: "ChartContextMenu",
  model: {
    prop: "visible",
    event: "change",
  },
  props: {
    visible: {
      type: Boolean,
      required: true,
      default: false,
    },
    contextData: {
      type: Object,
      default() {
        return {};
      },
    },
    disabledList: {
      type: Array,
      default() {
        return [];
      },
    },
    getParentNode: Function,
  },
  data() {
    const menuList = [
      {
        id: 0,
        title: this.$t('other.DisplayLimitHiddenValue'),
        method: "toggleValue",
      },
      {
        id: 1,
        title: this.$t('other.SortedinHightoLow'),
        method: "sortReverse",
      },
      {
        id: 2,
        title: this.$t('other.SortedinLowtoHigh'),
        method: "sort",
      },
    ];
    return {
      menuList,
    };
  },
  components: {},
  watch: {
    visible(n) {
      if (n) {
        this.initData();
        if (this.getParentNode) {
          // console.log(this.getParentNode)
          this.getParentNode().appendChild(this.$el);
        }
      }
      //  else {
      //   this.getParentNode().removeChild(this.$el);
      // }
    },
  },
  methods: {
    itemClick(obj) {
      // console.log(obj)
      if (
        obj.method &&
        "function" == typeof this[obj.method] &&
        !obj.disabled
      ) {
        this.close();
        this[obj.method](obj);
      }
    },
    close() {
      this.$emit("change", false);
    },
    toggleValue() {
      let { ref } = this.contextData;
      this.$emit("toggleLabel", ref);
    },
    toObjectList(labels, datas) {
      return labels.map((v, i) => ({ label: v, data: datas[i] }));
    },
    formatData(arr) {
      let xLabel = [];
      let sData = [];
      arr.forEach((v) => {
        xLabel.push(v.label);
        sData.push(v.data);
      });
      return {
        xLabel,
        sData,
      };
    },
    sort() {
      let { xLabel, data, ref } = this.contextData;
      let list = this.toObjectList(xLabel, data);
      list.sort((a, b) => a.data - b.data);
      this.$emit("updateChart", { ref, data: this.formatData(list) });
    },
    sortReverse() {
      let { xLabel, data, ref } = this.contextData;
      let list = this.toObjectList(xLabel, data);
      list.sort((a, b) => b.data - a.data);
      this.$emit("updateChart", { ref, data: this.formatData(list) });
    },
    initData() {
      this.menuList.forEach((v) => {
        v.disabled = this.disabledList.some((val) => val == v.id);
      });
    },
  },

  mounted() {
    this.initData();
  },
};
</script>

<style lang="less" scoped>
.chart-context-menu {
  z-index: 1;
  position: absolute;
  background: #c0c0c0;
  // border: 1px #055AC6 solid;
  border-radius: 4px;
  padding: 4px;
  .contain {
    position: relative;
    background: #c0c0c0;
    z-index: 100;
    .main-ul {
      display: flex;
      flex-direction: column;
      .main-li {
        position: relative;
        cursor: pointer;
        flex: 1;
        border: 1px #fff solid;
        .title {
          padding: 4px;
          color: #333;
          background: #f0f0f0;
          &:hover {
            background: #ccc;
          }
        }
        & ~ .main-li {
          margin-top: 2px;
        }
      }
      .disabled.disabled.disabled {
        cursor: not-allowed;
        color: #eee;
        background: #999;
      }
    }
  }
  .mask {
    position: fixed;
    left: 0;
    right: 0;
    top: 0;
    bottom: 0;
    background: transparent;
    z-index: 99;
  }
}
</style>